from django.db.models.fields import FieldDoesNotExist, CharField

def get_search_fieldname(model):
    # If model has field 'title' then use that, else use the first 
    # CharField on model.
    fieldname = ''
    try:
        model._meta.get_field_by_name('title')
        fieldname = 'title'
    except FieldDoesNotExist:
        for field in model._meta.fields:
            if isinstance(field, CharField):
                fieldname = field.name
                break                
    if not fieldname:
        raise RuntimeError, "Cannot determine fieldname"
    return fieldname



import pickle

from django.forms.widgets import Widget
from django.utils.safestring import mark_safe
from django.db.models.query import QuerySet
from django.core.urlresolvers import reverse

from simple_autocomplete.monkey import _simple_autocomplete_queryset_cache
from simple_autocomplete.utils import get_search_fieldname

class AutoCompleteWidget(Widget):
    input_type = 'autocomplete'
    url = None
    initial_display = None
    token = None
    model = None

    def __init__(self, url=None, initial_display=None, token=None, model=None, *args, **kwargs):
        """
        url: a custom URL that returns JSON with format [(value, label),(value,
        label),...].  

        initial_display: if url is provided then initial_display is the initial
        content of the autocomplete box, eg. "John Smith".

        token: an identifier to retrieve a cached queryset. Used internally.

        model: the model that the queryset objects are instances of. Used
        internally.
        """
        self.url = url
        self.initial_display = initial_display
        self.token = token
        self.model = model
        super(AutoCompleteWidget, self).__init__(*args, **kwargs)

    def render(self, name, value, attrs=None):
        if value is None:
            value = ''
      
        display = ''
        if self.url:
            url = self.url
            display = self.initial_display

        else:
            fieldname = get_search_fieldname(self.model)
            url = reverse('simple-autocomplete', args=[self.token])
            if value:            
                dc, dc, query = pickle.loads(_simple_autocomplete_queryset_cache[self.token])
                queryset = QuerySet(model=self.model, query=query)
                display = getattr(queryset.get(pk=value), fieldname)

        html = """
    <script type="text/javascript">
    $(document).ready(function(){

    $("#id_%s_helper").autocomplete({
        source: function(request, response){
            $.ajax({
                url: "%s",
                data: {q: request.term},
                success: function(data) {
                    if (data != 'CACHE_MISS')                    
                        response($.map(data, function(item) {
                            return {
                                label: item[1],
                                value: item[1],
                                real_value: item[0]
                            };
                        }));
                },
                dataType: "json"
            });
        },
        select: function(event, ui) { $('#id_%s').val(ui.item.real_value); },
        minLength: 3
    });

    });
    </script>

<input id="id_%s_helper" type="text" value="%s" />
<input name="%s" id="id_%s" type="hidden" value="%s" />""" % (name, url, name, name, display, name, name, value)
        return mark_safe(html)

    def value_from_datadict(self, data, files, name):
        return data.get(name, None)



import pickle
import hashlib

from django.forms.models import ModelChoiceField
from django.conf import settings
from django.forms.fields import Field

_simple_autocomplete_queryset_cache = {}

from simple_autocomplete.widgets import AutoCompleteWidget

def ModelChoiceField__init__(self, queryset, empty_label=u"---------", cache_choices=False,
             required=True, widget=None, label=None, initial=None,
             help_text=None, to_field_name=None, *args, **kwargs):
    if required and (initial is not None):
        self.empty_label = None
    else:
        self.empty_label = empty_label
    self.cache_choices = cache_choices

    # Monkey starts here
    # Do not apply patch to subclasses like ModelMultipleChoiceField
    if self.__class__ ==  ModelChoiceField:
        key = '%s.%s' % (queryset.model._meta.app_label, queryset.model._meta.module_name)
        if key in getattr(settings, 'SIMPLE_AUTOCOMPLETE_MODELS', []):
            pickled = pickle.dumps((
                queryset.model._meta.app_label, 
                queryset.model._meta.module_name, 
                queryset.query
            ))
            token = hashlib.md5(pickled).hexdigest()
            _simple_autocomplete_queryset_cache[token] = pickled
            widget = AutoCompleteWidget(token=token, model=queryset.model)
    # Monkey ends here        

    # Call Field instead of ChoiceField __init__() because we don't need
    # ChoiceField.__init__().
    Field.__init__(self, required, widget, label, initial, help_text,
                   *args, **kwargs)

    self.queryset = queryset
    self.choice_cache = None
    self.to_field_name = to_field_name

ModelChoiceField.__init__ = ModelChoiceField__init__



import simplejson
import pickle

from django.http import HttpResponse
from django.db.models.query import QuerySet
from django.db.models import get_model

from simple_autocomplete.monkey import _simple_autocomplete_queryset_cache
from simple_autocomplete.utils import get_search_fieldname

def get_json(request, token):
    """Return matching results as JSON"""
    result = []
    searchtext = request.GET['q']
    if len(searchtext) >= 3:
        pickled =_simple_autocomplete_queryset_cache.get(token, None)
        if pickled is not None:
            app_label, model_name, query = pickle.loads(pickled)
            model = get_model(app_label, model_name)
            queryset = QuerySet(model=model, query=query)
            fieldname = get_search_fieldname(model)
            di = {'%s__istartswith' % fieldname:searchtext}
            items = queryset.filter(**di).order_by(fieldname)[:10]
            for item in items:
                result.append((item.id, getattr(item, fieldname)))
        else:
            result = 'CACHE_MISS'
    return HttpResponse(simplejson.dumps(result))



import simplejson
import pickle
import hashlib

from django.db import models
from django.contrib.auth.models import User
from django import forms
from django.test import TestCase
from django.conf import settings
from django.test.client import Client as BaseClient, FakePayload, RequestFactory
from django.core.handlers.wsgi import WSGIRequest
from django.core.urlresolvers import reverse

from simple_autocomplete.widgets import AutoCompleteWidget
from simple_autocomplete.monkey import _simple_autocomplete_queryset_cache

class DummyModel(models.Model):
    user = models.ForeignKey(User, null=True, blank=True)
models.register_models('simple_autocomplete', DummyModel)

class EditDummyForm(forms.ModelForm):
    class Meta:
        model = DummyModel

class Client(BaseClient):
    """Bug in django/test/client.py omits wsgi.input"""

    def _base_environ(self, **request):
        result = super(Client, self)._base_environ(**request)
        result['HTTP_USER_AGENT'] = 'Django Unittest'
        result['HTTP_REFERER'] = 'dummy'
        result['wsgi.input'] = FakePayload('')
        return result

class TestCase(TestCase):

    def setUp(self):
        self.adam = User.objects.create_user('adam', 'adam@foo.com', 'password')
        self.eve = User.objects.create_user('eve', 'eve@foo.com', 'password')
        self.dummy = DummyModel()
        self.dummy.save()
        self.request = RequestFactory()
        self.client = Client()
    
    def test_monkey(self):
        # Are we using the autocomplete widget?
        form = EditDummyForm(self.request, instance=self.dummy)
        self.failUnless(isinstance(form.fields['user'].widget, AutoCompleteWidget))

    def test_json(self):
        # Find our token in cache
        for token, pickled in _simple_autocomplete_queryset_cache.items():
            app_label, model_name, dc = pickle.loads(pickled)
            if (app_label == 'auth') and (model_name == 'user'):
                break

        url = reverse('simple-autocomplete', args=[token])
        response = self.client.get(url, {'q':'ada'})
        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.content, """[[1, "adam"]]""")
